@@ -1,12 +1,14 @@
-#include <kwm.h>
+//#include <kwm.h>
 #include <kapp.h>
 #include <qkeycode.h>
 #include "stringres.h"
 
 #include "Login.h"
+#include "kglobal.h"
+#include "kconfig.h"
 
 #ifndef rcsid
-static const char rcsid[]="$Id: Login.cpp,v 2.6 1999/11/07 15:42:04 ral Exp $";
+static const char rcsid[]="$Id: Login.cpp,v 2.8 2001/04/08 12:24:47 ral Exp $";
 #endif
 
 #define Inherited LoginData
@@ -15,10 +17,25 @@ CLogin::CLogin(QWidget* parent, const char* name )
     :Inherited( parent, name )
 {
     setCaption( CStringRes::get_string(45));
-    KWM::setIcon(winId(),kapp->getIcon());
-    KWM::setMiniIcon(winId(),kapp->getMiniIcon());
+    setIcon(kapp->icon());
+    //    setMiniIcon(kapp->miniicon());
+    //KWM::setMiniIcon(winId(),kapp->getMiniIcon());
+
+    KConfig * k = KGlobal::config();
+    k->setGroup("last_login");
+    QString user,host,port;
+    user = k->readEntry("user");
+    host = k->readEntry("host");
+    port = k->readEntry("port");
     
-    PortEdit->setText("3306");
+    if (port.length() > 0) 
+	PortEdit->setText(port);
+    else
+	PortEdit->setText("3306");
+    if (host.length() > 0)
+	HostEdit->setText(host);
+    if (user.length() > 0)
+	UserEdit->setText(user);
     HostEdit->setFocus();
 }
 
