@@ -1,6 +1,6 @@
 // -*-Mode: C++;-*-
 /*
- * $Id: Login.h,v 2.2 1999/11/07 15:42:04 ral Exp $;
+ * $Id: Login.h,v 2.3 2000/02/27 13:36:47 ral Exp $;
  * Include file for login-window
  */
 #ifndef CLogin_included
@@ -15,10 +15,10 @@ class CLogin : public LoginData
 public:
     CLogin(QWidget* parent = NULL,const char* name = NULL);
     virtual ~CLogin();
-    const char*get_host(){return HostEdit->text();}
-    const char*get_pass(){return PassEdit->text();}
-    const char*get_user(){return UserEdit->text();}
-    const char*get_port(){return PortEdit->text();}
+    const char*get_host()const{return HostEdit->text();}
+    const char*get_pass()const{return PassEdit->text();}
+    const char*get_user()const{return UserEdit->text();}
+    const char*get_port()const{return PortEdit->text();}
 
 protected slots:
     void focus_user();
