@@ -1,6 +1,6 @@
 // -*-Mode: C++;-*-
 /*
- * $Id: MainApp.h,v 2.5 1999/11/08 03:32:34 ral Exp $
+ * $Id: MainApp.h,v 2.10 2001/04/08 12:24:47 ral Exp $
  */
 
 #ifndef CMainApp_included
@@ -13,20 +13,21 @@
 
 class CMainWindow;
 
-class CMainApp : public KTMainWindow,virtual public CSendMessage
+class CMainApp : public KMainWindow,virtual public CSendMessage
 {
     Q_OBJECT
 
 public:
-    CMainApp ( const char* name = NULL );
+    CMainApp ( const char* name = NULL,bool _first = false );
     
     virtual ~CMainApp();
     virtual void append_message(const char*text);
     virtual void append_message(const std::string&message);
     
-private:
+protected:
     QPopupMenu*file;
     QPopupMenu*db_menu;
+    QPopupMenu*setup_menu;
     int logf;
     QString logfile;
     QFile filepointer;
@@ -39,7 +40,12 @@ protected slots:
     void set_LogFileName();
     void connected_to_server(const char*);
     void disconnected_from_server();
+    bool queryExit();
+    bool queryClose();
+    void layout_setup();
+
 public slots:
     void windowClose();
+signals:
 };
 #endif // CMainApp_included
