@@ -19,6 +19,8 @@
 #include "e_patternpair.h"
 #include "MultiTextInputData.h"
 
+class QFrame;
+
 class CMultiTextInput : public CMultiTextInputData
 {
     Q_OBJECT
@@ -28,18 +30,19 @@ public:
     void build_widgets(e_PatternList*which);
     PatternList*get_entries(bool emptys_too=FALSE);
 
-private:
+protected:
     QList<QWidget>*EditList;
-    QBoxLayout * EditLayout;
+    QVBoxLayout * EditLayout;
+    QVBoxLayout * ScrollLayout;
+    QFrame*FrameWidget;
+    void ResizeFrame();
+
     int min_x;
     int min_y;
 
-private slots:
-    void change_focus();
-
 protected slots:
-    void keyPressEvent(QKeyEvent*e);
+    virtual void keyPressEvent(QKeyEvent*e);
     virtual void resizeEvent ( QResizeEvent * );
-
+    virtual void change_focus();
 };
 #endif // CMultiTextInput_included
