@@ -5,6 +5,7 @@
 
 #include "TextViewData.h"
 #include <qtimer.h>
+#include "smart_pointer.h"
 
 class CMySql;
 
@@ -13,17 +14,20 @@ class CStatusView : public CTextViewData
     Q_OBJECT
 
 public:
-
-    CStatusView(CMySql*_sql_server,QWidget* parent = NULL,const char* name = NULL);
+    CStatusView(smart_pointer<CMySql>& _sql_server,QWidget* parent = NULL,const char* name = NULL);
     virtual ~CStatusView();
     
 protected:
-    CMySql*sql_server;
+    smart_pointer<CMySql> sql_server;
     QTimer*timer;
+    int m_Timeout;
     void get_status();
     
-private slots:
-    void RefreshOutput();
+protected slots:
+    virtual void RefreshOutput();
+    virtual void newTimeout(int aValue);
+    virtual void clear(){};
+    virtual void newFont();
 
 public slots:
     void hide();
