@@ -1,7 +1,7 @@
 #include "TextInput.h"
 
 #ifndef rcsid
-static const char rcsid[]="$Id: TextInput.cpp,v 2.4 1999/11/28 15:40:10 ral Exp $";
+static const char rcsid[]="$Id: TextInput.cpp,v 2.5 2001/04/08 12:24:47 ral Exp $";
 //const char*textinput_rcs(){return rcsid;}
 #endif
 
@@ -28,9 +28,9 @@ void CTextInput::accept_field()
     QDialog::accept();
 }
 
-void CTextInput::inputChanged(const char* text)
+void CTextInput::inputChanged(const QString&text)
 {
-    if (strlen(text) > 0)
+    if (!text.isEmpty())
 	OkButton->setEnabled(TRUE);
     else
 	OkButton->setEnabled(FALSE);
