@@ -1,6 +1,6 @@
 // -*-Mode: C++;-*- 
 /*
- * $Id: ThreadView.h,v 2.2 1999/11/07 15:42:05 ral Exp $
+ * $Id: ThreadView.h,v 2.5 2001/04/08 12:24:47 ral Exp $
  * Include file for the thread-view-object
  */
 
@@ -9,6 +9,7 @@
 
 #include "ListViewData.h"
 #include "qtimer.h"
+#include "smart_pointer.h"
 
 class CMySql;
 
@@ -18,18 +19,19 @@ class CThreadView : public CListViewData
 
 public:
 
-    CThreadView(CMySql*_sql_server,QWidget* parent = NULL,const char* name = NULL);
-
+    CThreadView(smart_pointer<CMySql>& _sql_server,QWidget* parent = NULL,const char* name = NULL);
     virtual ~CThreadView();
 
 protected slots:
     void RefreshList();
     void KillThread();
+    void newTimeout(int aValue);
+    void NewFixedFont();
 
 protected:
-    CMySql*sql_server;
+    smart_pointer<CMySql> sql_server;
     QTimer*ResetTimer;
-    QPushButton*KillButton;
+    int m_Timeout;
 
 public slots:
     void hide();
