@@ -15,6 +15,7 @@
 //#include "mysqlw.h"
 
 #include "stringlist.h"
+#include "smart_pointer.h"
 
 class CMySql;
 
@@ -23,15 +24,15 @@ class CChangeTableName : public CChangeTableNameData
     Q_OBJECT
 
 public:
-    CChangeTableName(CMySql*_sql_server,QWidget* parent = NULL,const char* name = NULL);
+    CChangeTableName(smart_pointer<CMySql>& _sql_server,QWidget* parent = NULL,const char* name = NULL);
     virtual ~CChangeTableName();
 protected slots:
     virtual void change_name();
-    virtual void name_changed(const char*);
+    virtual void name_changed(const QString&);
     virtual void db_selected(int);
     virtual void table_selected(int);
 protected:
-    CMySql*sql_server;
+    smart_pointer<CMySql> sql_server;
     stringlist db_list;
     stringlist table_list;
 };
