@@ -1,6 +1,6 @@
 // -*-Mode: C++;-*- 
 /*
- * $Id: createtable.h,v 2.9 1999/11/07 15:42:05 ral Exp $
+ * $Id: createtable.h,v 2.11 2001/04/08 14:00:03 ral Exp $
  */
 #ifndef CCreateTable_included
 #define CCreateTable_included
@@ -19,13 +19,13 @@ class CCreateTable : public CModifyTable
 
 public:
 
-    CCreateTable(CMySql*_sql_server,QWidget* parent = NULL,const char* name = NULL);
+    CCreateTable(smart_pointer<CMySql>& _sql_server,QWidget* parent = NULL,const char* name = NULL);
     virtual ~CCreateTable();
 
 protected slots:
-    virtual void TableTextChanged(const char*);
+    virtual void TableTextChanged(const QString&text);
     virtual void TableTextActivate();
-    virtual void SetDBString(const char*);
+    virtual void SetDBString(const QString&);
     virtual void AddColumn();
     virtual void Delete_Column();
     virtual void CreateTable();
