@@ -1,7 +1,7 @@
 // -*-Mode: C++;-*- 
 
 /*
- * $Id: deltabledlg.h,v 2.5 1999/11/07 15:42:05 ral Exp $
+ * $Id: deltabledlg.h,v 2.7 2001/04/16 13:01:09 ral Exp $
  * Include file for delete table dlg
  */
 
@@ -10,6 +10,7 @@
 
 #include "deltabledata.h"
 #include "stringlist.h"
+#include "smart_pointer.h"
 
 class CMySql;
 
@@ -17,16 +18,17 @@ class CDeleteTableDlg : public CDeleteTableDlgData
 {
     Q_OBJECT
 public:
-    CDeleteTableDlg(CMySql*_sql_server,QWidget* parent = NULL,const char* name = NULL);
+    CDeleteTableDlg(smart_pointer<CMySql>& _sql_server,QWidget* parent = NULL,const char* name = NULL);
     virtual ~CDeleteTableDlg();
 
 protected:
     stringlist db_list;
-    CMySql*sql_server;
+    smart_pointer<CMySql> sql_server;
 
 protected slots:
     void HilightDB(int);
     void DelTable();
+    void TableHilight(int);
     const char*get_error_head();
 };
 #endif // CDeleteTableDlg_included
