@@ -7,17 +7,23 @@
 
 #include <qstring.h>
 #include <qmsgbox.h>
-#include <kwm.h>
+#include <klocale.h>
 #include <kapp.h>
+#include <kglobal.h>
 
 #include "globals.h"
 #include "granttable.h"
 #include "grantedit.h"
 #include "my_sql.h"
+#include "stringres.h"
+#include "settings.h"
+#include "TabView.h"
+#include "qlistview.h"
+
 #include "stringres.h"
 
 #ifndef rcsid
-static const char rcsid[]="$Id: granttable.cpp,v 2.10 1999/11/07 16:12:50 ral Exp $";
+static const char rcsid[]="$Id: granttable.cpp,v 2.19 2001/04/29 11:14:07 ral Exp $";
 //const char*granttable_rcs(){return rcsid;}
 #endif
 
@@ -43,27 +49,30 @@ const char*CGrantTable::No="N";
 
 #define Inherited CGrantTableData
 
-CGrantTable::CGrantTable(CMySql*_sql_server,QWidget* parent,const char* name)
+CGrantTable::CGrantTable(smart_pointer<CMySql>& _sql_server,QWidget* parent,const char* name)
     :Inherited( parent, name )
 {
     sql_server = _sql_server;
     QString caption;
-	User_Title=i18n("User:");
-	DB_Title=i18n("Database:");
-	P_Title=i18n("Password:");
-	P2_Title=i18n("Retype Pass:");
-
-    caption = CStringRes::get_string(125);
-    caption += i18n(" - Edit grants");
+    User_Title=i18n(CStringRes::get_string(25));
+    DB_Title=i18n(CStringRes::get_string(26));
+    P_Title=i18n(CStringRes::get_string(27));
+    P2_Title=i18n(CStringRes::get_string(28));
+    caption = i18n(CStringRes::get_string(125));
+    caption += i18n(CStringRes::get_string(29));
     setCaption(caption);
-    KWM::setIcon(winId(),kapp->getIcon());
-    KWM::setMiniIcon(winId(),kapp->getMiniIcon());
+    setIcon(kapp->icon());
+    //setMiniIcon(kapp->miniIcon());
+    FontChanged();
     DisableButtons();
     ReadHostTable();
+    connect(&GlobalSettings,SIGNAL(fixfont_changed()),this,SLOT(FontChanged()));
+    FontChanged();
 }
 
 CGrantTable::~CGrantTable()
 {
+    debug("Granttable: destruktor\n");
 }
 
 void CGrantTable::Refresh()
@@ -80,57 +89,57 @@ void CGrantTable::ReadHostTable()
 	return;
     erg.push_back(std::string(Host_Table));
     erg.push_back(std::string(DB_Table));
-    i = sql_server->fetch_rows(DB_Name,Host_Table,Host_Table,&erg);
-    if (erg.size() > 0 && i==0) {
-	stringlist::iterator iter = erg.begin();
-	for (;iter!=erg.end();++iter) {
-	    HostList->insertItem(iter->c_str());
+    HostList->clear();
+    list_vector result;
+    i = sql_server->fetch_rows(DB_Name,Host_Table,Host_Table,&erg,result,0,0,0,true);
+    if (result.size() > 0 && i==0) {
+        QListViewItem * item = 0;
+        list_vector::iterator result_iter = result.begin();
+	stringlist::iterator iter;
+	for (;result_iter!=result.end();++result_iter) {
+            iter = result_iter->begin();
+            item = new QListViewItem(HostList,item);
+            int count = 0;
+            for (;iter!=result_iter->end();++iter) {
+                item->setText(count,iter->c_str());
+                ++count;
+                if (count == 2)break;
+            }
 	}
+    } else {
     }
-    if (HostList->count() > 0 && i == 0)
-	HostList->setCurrentItem(0);
+    HostList->setCurrentItem(HostList->firstChild());
     if (i == 0)
 	AddHostButton->setEnabled(TRUE);
 }
 
-void CGrantTable::hosthilighted(int i)
+void CGrantTable::hosthilighted(QListViewItem*hostitem)
 {
-    QString s = HostList->text(i);
-    QString t;
-    int j = s.find("\t");
-    if (j > -1) {
-	t = s.left(j);
-	t = t.stripWhiteSpace();
-	if (t == "%")
-	    t = "%%";
-    }
-    else
-	t = s;
+    if (hostitem==0)
+        return;
+    QString t = hostitem->text(0);
     ReadUserTable(t);
     ModifyHostButton->setEnabled(TRUE);
-    if (UserList->count() > 0)
+    if (UserList->childCount() > 0)
 	DelHostButton->setEnabled(FALSE);
     else
-	DelHostButton->setEnabled(TRUE);
+        DelHostButton->setEnabled(TRUE);
 }
 
-void CGrantTable::userhilighted(int i)
+void CGrantTable::userhilighted(QListViewItem*useritem)
 {
-    QString s = UserList->text(i);
-    QString t;
-    int j = s.find("\t");
-    if (j > -1) {
-	t = s.left(j);
-	s = s.mid(j+1,s.length());
-	t = t.stripWhiteSpace();
-	s = s.stripWhiteSpace();
-	ReadDBTable(t,s);
-    }
+    if (useritem==0)
+        return;
+    QString t,s;
+    t = useritem->text(0);
+    s = useritem->text(1);
+    ReadDBTable(t,s);
+
     ModifyUserButton->setEnabled(TRUE);
-    if (DBList->count() == 0)
+    if (DBList->childCount() == 0)
 	DelUserButton->setEnabled(TRUE);
     else
-	DelUserButton->setEnabled(FALSE);
+        DelUserButton->setEnabled(FALSE);
 }
 
 void CGrantTable::show()
@@ -138,29 +147,39 @@ void CGrantTable::show()
     QDialog::show();
 }
 
-void CGrantTable::ReadUserTable(const char*Host)
+void CGrantTable::ReadUserTable(QString&Host)
 {
-    stringlist erg ;
+    stringlist erg;
     PatternList Patterns;
     int i;
 
     UserList->clear();
-    if (!Host || !sql_server)
+    if (!sql_server)
 	return;
     erg.push_back(Host_Table);
     erg.push_back(User_Table);
+    list_vector result;
     Patterns.push_back(CPatternPair(Host_Table,Host));
-    i = sql_server->fetch_rows(DB_Name,User_Table,User_Table,&erg,0,0,&Patterns);
+    i = sql_server->fetch_rows(DB_Name,User_Table,User_Table,&erg,result,0,0,&Patterns,true);
 
-    if (erg.size()) {
-	stringlist::iterator iter = erg.begin();
-	for (;iter != erg.end();++iter) {
-	    UserList->insertItem(iter->c_str());
-	}
+    if (result.size()) {
+        QListViewItem* item=0;
+        list_vector::iterator result_iter=result.begin();
+	stringlist::iterator iter;
+        for (;result_iter!=result.end();++result_iter) {
+            iter = result_iter->begin();
+            item = new QListViewItem(UserList,item);
+            int count = 0;
+            for (;iter!=result_iter->end();++iter) {
+                item->setText(count,iter->c_str());
+                ++count;
+                if (count == 2)break;
+            }
+        }
     }
-    if (UserList->count()>0 && i == 0)
-	UserList->setCurrentItem(0);
-    else {
+    if (UserList->childCount()>0 && i == 0) {
+	UserList->setCurrentItem(UserList->firstChild());
+    } else {
 	ModifyUserButton->setEnabled(FALSE);
 	DelUserButton->setEnabled(FALSE);
     }
@@ -170,7 +189,7 @@ void CGrantTable::ReadUserTable(const char*Host)
 	AddUserButton->setEnabled(FALSE);
 }
 
-void CGrantTable::ReadDBTable(const char*Host,const char*User)
+void CGrantTable::ReadDBTable(QString&Host,QString&User)
 {
     stringlist erg;
     PatternList Patterns;
@@ -178,29 +197,36 @@ void CGrantTable::ReadDBTable(const char*Host,const char*User)
     int i;
 
     DBList->clear();
-    if (!Host || !User || !sql_server)
+    if (!sql_server)
 	return;
 
     erg.push_back(Host_Table);
     erg.push_back(User_Table);
     erg.push_back(DB_Table);
+    list_vector result;
 
     Patterns.push_back(CPatternPair(Host_Table,Host));
     Patterns.push_back(CPatternPair(User_Table,User));
 
-    i = sql_server->fetch_rows(DB_Name,DB_Table,DB_Table,&erg,0,0,&Patterns);
-    if (erg.size()) {
-	stringlist::iterator iter = erg.begin();
-	for (;iter!=erg.end();++iter) {
-	    DBList->insertItem(iter->c_str());
-	}
-	if (erg.size() > 0) {
-	    DBList->setCurrentItem(0);
-	}
-	else {
-	    ModifyDbButton->setEnabled(FALSE);
-	    DelDbButton->setEnabled(FALSE);
-	}
+    i = sql_server->fetch_rows(DB_Name,DB_Table,DB_Table,&erg,result,0,0,&Patterns,true);
+    if (result.size() && i==0) {
+        QListViewItem*item = 0;
+        list_vector::iterator result_iter = result.begin();
+	stringlist::iterator iter;
+        for (;result_iter!=result.end();++result_iter) {
+            iter = result_iter->begin();
+            item = new QListViewItem(DBList,item);
+            int count = 0;
+            for (;iter!=result_iter->end();++iter) {
+                item->setText(count,iter->c_str());
+                ++count;
+                if (count==3)break;
+            }
+        }
+        DBList->setCurrentItem(DBList->firstChild());
+    } else {
+        ModifyDbButton->setEnabled(FALSE);
+        DelDbButton->setEnabled(FALSE);        
     }
     if (i == 0) 
 	AddDbButton->setEnabled(TRUE);
@@ -221,10 +247,16 @@ void CGrantTable::DisableButtons()
     DelDbButton->setEnabled(FALSE);
 }
 
-void CGrantTable::dbhighlighted(int)
+void CGrantTable::dbhilighted(QListViewItem*)
 {
-    ModifyDbButton->setEnabled(TRUE);
-    DelDbButton->setEnabled(TRUE);
+    QListViewItem*item = DBList->currentItem();
+    if (item) {
+        ModifyDbButton->setEnabled(TRUE);
+        DelDbButton->setEnabled(TRUE);
+    } else {
+        ModifyDbButton->setEnabled(FALSE);
+        DelDbButton->setEnabled(FALSE);
+    }
 }
 
 void CGrantTable::ModifyHost()
@@ -255,7 +287,7 @@ void CGrantTable::ModifyHost()
     if (db == "%")
 	(*Patterns)[1].set_pattern("%%");
     
-    i = sql_server->fetch_rows(DB_Name,Host_Table,0,&erg,0,0,Patterns);
+    i = sql_server->fetch_rows(DB_Name,Host_Table,0,&erg,0,0,Patterns,true);
 
     if (i == -1 || Patterns->size() == 0) {
 	Error();
@@ -264,7 +296,7 @@ void CGrantTable::ModifyHost()
     }
     Patterns->insert(Patterns->begin(),CPatternPair(DB_Table,db));
     Patterns->insert(Patterns->begin(),CPatternPair(Host_Table,host));
-    if (UserList->count() > 0)
+    if (UserList->childCount() > 0)
 	(*Patterns)[0].setIsNumeric(TRUE);
     erg.clear();
     erg.push_back(Host_Title);
@@ -299,7 +331,7 @@ void CGrantTable::ModifyUser()
     PatternList*modify = 0;
     CPatternPair pat2;
     CGrantEdit edit(this);
-    edit.setCaption(i18n("Edit User-Rights"));
+    edit.setCaption(CStringRes::get_string(30));//i18n("Edit User-Rights"));
     int i = 0;
     int j = 0;
 
@@ -315,11 +347,17 @@ void CGrantTable::ModifyUser()
 	return;
 
     Host = (*Patterns)[0].get_pattern();
+    if (Host == "%")
+	(*Patterns)[0].set_pattern("%%");
     User = (*Patterns)[1].get_pattern();
+    if (User == "%")
+	(*Patterns)[1].set_pattern("%%");
 
-    i = sql_server->fetch_rows(DB_Name,User_Table,0,&erg,0,0,Patterns);
+    i = sql_server->fetch_rows(DB_Name,User_Table,0,&erg,0,0,Patterns,true);
     if (i == -1 || Patterns->size() == 0) {
 	Error();
+        if (Patterns->size()==0) 
+            qWarning("Patternsize == 0");
 	delete Patterns;
 	return;
     }
@@ -330,8 +368,7 @@ void CGrantTable::ModifyUser()
     (*Patterns)[0].setIsPassword(TRUE);
 
     Patterns->insert(Patterns->begin(),CPatternPair(User_Table,User));
-    (*Patterns)[0].setIsNumeric( (DBList->count() > 0) );
-    
+    (*Patterns)[0].setIsNumeric( (DBList->childCount() > 0) );
     Patterns->insert(Patterns->begin(),CPatternPair(Host_Table,Host));
     (*Patterns)[0].setIsNumeric(TRUE);
 
@@ -342,7 +379,7 @@ void CGrantTable::ModifyUser()
     erg.push_back(P2_Title);
     edit.set_entries(Patterns,&erg);
     erg.clear();
-    j = DBList->count() > 0?0:1;
+    j = DBList->childCount() > 0?0:1;
     if (edit.exec()) {
 	modify = edit.get_entries();
 	if (modify && modify->size()>0) {
@@ -351,19 +388,24 @@ void CGrantTable::ModifyUser()
 	    if (p1 != p2) {
 		pass_error();
 	    } else {
-		if (p1 == "")
+		if (p1 == "") {
 		    Old_Pass = "";
-		else if (p1 != Old_Pass) {
+		} else if (p1 != Old_Pass) {
 		    sql_server->scramble_password(p1);
 		}
+		pat2 = (*modify)[0];
 		modify->erase(modify->begin());
 		modify->erase(modify->begin());
-		modify->insert(modify->begin(),CPatternPair(Pass_Entry,p1));
+		modify->erase(modify->begin());
+		if (p1 == string("user"))
+		    modify->insert(modify->begin(),pat2);
+                modify->insert(modify->begin(),CPatternPair(Pass_Entry,p1));
 		Patterns->clear();
 		Patterns->push_back(CPatternPair(Host_Table,Host));
 		Patterns->push_back(CPatternPair(User_Table,User));
 		i = sql_server->update_row(DB_Name,User_Table,modify,Patterns);
 		UpdateGrants(i);
+		ReadUserTable(Host);
 	    }
 	}
     }
@@ -383,33 +425,23 @@ void CGrantTable::ModifyDb()
     PatternList*modify = 0;
     
     CGrantEdit edit(this);
-    edit.setCaption(i18n("Edit DB-Rights"));
-    
-    int i = DBList->currentItem();
-    int j = 0;
-    if (i<0)
-	return;
-    dbentry = DBList->text(i);
-    j = dbentry.find("\t");
-    if (j < 0)
-	return;
+    edit.setCaption(CStringRes::get_string(31));//i18n("Edit DB-Rights"));
 
     append_Sel(&erg);
-
     Patterns = break_dbstring();
     if (Patterns == 0)
-	return;
+        return;
 
     hostentry = (*Patterns)[0].get_pattern();
     userentry = (*Patterns)[1].get_pattern();
     dbentry = (*Patterns)[2].get_pattern();
 
-    i = sql_server->fetch_rows(DB_Name,DB_Table,0,&erg,0,0,Patterns);
+    int i = sql_server->fetch_rows(DB_Name,DB_Table,0,&erg,0,0,Patterns,true);
 
     if (i == -1 || Patterns->size() == 0) {
-	Error();
-	delete Patterns;
-	return;
+        Error();
+        delete Patterns;
+        return;
     }
     Patterns->insert(Patterns->begin(),CPatternPair(DB_Table,dbentry));
     Patterns->insert(Patterns->begin(),CPatternPair(User_Table,userentry));
@@ -423,19 +455,23 @@ void CGrantTable::ModifyDb()
     edit.set_entries(Patterns,&erg);
 
     if (edit.exec()) {
-	modify = edit.get_entries();
-	if (modify && modify->size() > 0) {
-	    Patterns->clear();
-	    Patterns->push_back(CPatternPair(Host_Table,hostentry));
-	    Patterns->push_back(CPatternPair(User_Table,userentry));
-	    Patterns->push_back(CPatternPair(DB_Table,dbentry));
-	    i = sql_server->update_row(DB_Name,DB_Table,modify,Patterns);
-	    UpdateGrants(i);
-	}
+ 	modify = edit.get_entries();
+ 	if (modify && modify->size() > 0) {
+            Patterns->clear();
+ 	    Patterns->push_back(CPatternPair(Host_Table,hostentry));
+ 	    Patterns->push_back(CPatternPair(User_Table,userentry));
+ 	    Patterns->push_back(CPatternPair(DB_Table,dbentry));
+ 	    i = sql_server->update_row(DB_Name,DB_Table,modify,Patterns);
+ 	    UpdateGrants(i);
+            QListViewItem*item = UserList->selectedItem();
+            if (item) {
+                userhilighted(item);
+            }
+ 	}
     }
     delete Patterns;
     if (modify)
-	delete modify;
+        delete modify;
 }
 
 void CGrantTable::append_Sel(stringlist*List)
@@ -471,7 +507,7 @@ void CGrantTable::AddHost()
     int i;
     CGrantEdit edit(this);
     Patterns=new PatternList;
-    edit.setCaption(i18n("Add Host"));
+    edit.setCaption(GET_TEXT(32));
     Patterns->push_back(CPatternPair(Host_Table,""));
     Patterns->push_back(CPatternPair(DB_Table,""));
     
@@ -502,16 +538,14 @@ void CGrantTable::AddUser()
     QString Host = "";
     PatternList*Patterns = 0;
     stringlist erg;
-
     CGrantEdit edit(this);
     std::string p1, p2;
-    int j = 0;
     int i = 0;
-    edit.setCaption(i18n("Add User"));
+    edit.setCaption(GET_TEXT(33));
     Patterns = break_hoststring();
     PatternList::iterator p_iter;
     if (Patterns == 0)
-	return;
+        return;
     (*Patterns)[0].setIsNumeric(TRUE);
     Host = (*Patterns)[0].get_pattern();
     p_iter = Patterns->begin();
@@ -519,9 +553,9 @@ void CGrantTable::AddUser()
     Patterns->erase(p_iter);
     Patterns->push_back(CPatternPair(User_Table,""));
     Patterns->push_back(CPatternPair("pass",""));
-		PatternIterator p_iter2;
-		p_iter2 = Patterns->end();
-		--p_iter2;
+    PatternIterator p_iter2;
+    p_iter2 = Patterns->end();
+    --p_iter2;
     p_iter2->setIsPassword(TRUE);
     Patterns->push_back(CPatternPair("pass2",""));
 		p_iter2 = Patterns->end();
@@ -566,9 +600,9 @@ void CGrantTable::AddUser()
 		i = sql_server->insert_row(DB_Name,User_Table,Patterns);
 		UpdateGrants(i);
 		if (i > 0) {
-		    j = HostList->currentItem();
-		    if (j > -1) {
-			hosthilighted(j);
+                    QListViewItem*item = HostList->selectedItem();
+		    if (item) {
+			hosthilighted(item);
 		    }
 		}
 	    }
@@ -579,98 +613,89 @@ void CGrantTable::AddUser()
 
 void CGrantTable::AddDb()
 {
-    int i = 0;
-    int j = 0;
-    QString Host = "";
-    QString User = "";
-    QString Title = i18n("Add Db to user ");
-    stringlist titles;
-    PatternList*Patterns;
-    CGrantEdit edit(this);
+        int i = 0;
+        QString Host = "";
+        QString User = "";
+        QString Title = GET_TEXT(34);
+        stringlist titles;
+        PatternList*Patterns;
+        CGrantEdit edit(this);
 
-    Patterns = break_userstring();
-    if (Patterns == 0)
-	return;
+        Patterns = break_userstring();
+        if (Patterns == 0)
+            return;
 
-    Host = (*Patterns)[0].get_pattern();
-    (*Patterns)[0].setIsNumeric(TRUE);
-    User = (*Patterns)[1].get_pattern();
-    (*Patterns)[1].setIsNumeric(TRUE);
-    Patterns->push_back(CPatternPair(DB_Table,""));
-    append_Add(Patterns);
-    Title+=User;
-    edit.setCaption(Title);
+        Host = (*Patterns)[0].get_pattern();
+        (*Patterns)[0].setIsNumeric(TRUE);
+        User = (*Patterns)[1].get_pattern();
+        (*Patterns)[1].setIsNumeric(TRUE);
+        Patterns->push_back(CPatternPair(DB_Table,""));
+        append_Add(Patterns);
+        Title+=User;
+        edit.setCaption(Title);
 
-    titles.push_back(Host_Title);
-    titles.push_back(User_Title);
-    titles.push_back(DB_Title);
-    edit.set_entries(Patterns,&titles);
+        titles.push_back(Host_Title);
+        titles.push_back(User_Title);
+        titles.push_back(DB_Title);
+        edit.set_entries(Patterns,&titles);
     
-    delete Patterns; Patterns = 0;
+        delete Patterns; Patterns = 0;
 
-    if (!edit.exec())
-	return;
-    Patterns = edit.get_entries();
-    if (!Patterns)
-	return;
-    Patterns->insert(Patterns->begin(),CPatternPair(Host_Table,Host));
-    Patterns->insert(Patterns->begin(),CPatternPair(User_Table,User));
-    i = sql_server->insert_row(DB_Name,DB_Table,Patterns);
-    UpdateGrants(i);
-    if (i > 0) {
-	j = UserList->currentItem();
-	if (j > -1)
-	    userhilighted(j);
-    }
+        if (!edit.exec())
+            return;
+        Patterns = edit.get_entries();
+        if (!Patterns)
+            return;
+        Patterns->insert(Patterns->begin(),CPatternPair(Host_Table,Host));
+        Patterns->insert(Patterns->begin(),CPatternPair(User_Table,User));
+        i = sql_server->insert_row(DB_Name,DB_Table,Patterns);
+        UpdateGrants(i);
+        if (i > 0) {
+            userhilighted(UserList->selectedItem());
+        }
 }
 
 void CGrantTable::pass_error()
 {
-    QString msg = i18n("You had typed in different passwords");
+    QString msg = GET_TEXT(35);
     msg+=",\n";
-    msg+=i18n("please try again");
-    QMessageBox::warning(this,
-			  i18n("Error with password"),
-			  msg );
+    msg+=GET_TEXT(36);
+    QMessageBox::warning(this,GET_TEXT(37), msg );
 }
 
 void CGrantTable::user_error()
 {
-    QMessageBox::warning(this,
-			  i18n("Error with Username"),
-			  i18n("You had not typed in a username"));
+    QMessageBox::warning(this,GET_TEXT(38),GET_TEXT(39));
 }
 
 void CGrantTable::DeleteUser()
 {
-    if (DBList->count() != 0)
+    if (DBList->childCount() != 0)
 	return;
     PatternList*KeyList=0;
-
     int i = Ask_delete();
     if (i != QMessageBox::Yes)
-	return;
+        return;
 
     KeyList = break_userstring();
     if (!KeyList)
-	return;
-
+        return;
     i = sql_server->delete_row(DB_Name,User_Table,KeyList);
 
     delete KeyList;
 
     if (i < 0) {
-	Error();
-	return;
+        Error();
+        return;
     }
-    i = HostList->currentItem();
-    if (i > -1)
-	hosthilighted(i);
+    QListViewItem*item = HostList->selectedItem();
+    if (item)
+        hosthilighted(item);
 }
 
 void CGrantTable::DeleteHost()
 {
-    if (UserList->count() != 0)
+    if (UserList->childCount() != 0)
 	return;
     PatternList*KeyList=0;
     int i = Ask_delete();
@@ -690,7 +715,6 @@ void CGrantTable::DeleteHost()
 
 void CGrantTable::DeleteDb()
 {
-    int j = 0;
     PatternList*KeyList = 0;
     int i = Ask_delete();
     if (i != QMessageBox::Yes)
@@ -704,77 +728,51 @@ void CGrantTable::DeleteDb()
 	Error();
 	return;
     }
-    j = UserList->currentItem();
-    if (j > -1)
-	userhilighted(j);
+    userhilighted(UserList->selectedItem());
 }
 
 PatternList* CGrantTable::break_hoststring()
 {
-    PatternList*KeyList = 0;
-    int j = HostList->currentItem();
-    if (j < 0)
-	return KeyList;
-    QString db = HostList->text(j);
-    j = db.find("\t");
-    if (j < 0)
-	return KeyList;
-    QString Host = db.left(j);
-    Host = Host.stripWhiteSpace();
-    db = db.mid(j+1,db.length());
-    db = db.stripWhiteSpace();
-    
-    KeyList = new PatternList;
+    QListViewItem * item = HostList->selectedItem();
+    if (!item)
+        return 0;
+    QString db,Host;
+    PatternList*KeyList = new PatternList;
+    db = item->text(1);
+    Host=item->text(0);
     KeyList->push_back(CPatternPair(Host_Table,Host));
     KeyList->push_back(CPatternPair(DB_Table,db));
-    
     return KeyList;
 }
 
 PatternList* CGrantTable::break_userstring()
 {
-    int j = UserList->currentItem();
-    PatternList*KeyList = 0;
-    if (j < 0)
-	return KeyList;
-    QString user = UserList->text(j);
-    j = user.find("\t");
-    if (j < 0)
-	return KeyList;
-    QString host = user.left(j);
-    host = host.stripWhiteSpace();
-    user = user.mid(j+1,user.length());
-    user = user.stripWhiteSpace();
-
-    KeyList = new PatternList;
-    KeyList->push_back(CPatternPair(Host_Table,host));
-    KeyList->push_back(CPatternPair(User_Table,user));
-
-    return KeyList;
+   QListViewItem * item = UserList->selectedItem();
+   if (!item)
+       return 0;
+   PatternList*KeyList = 0;
+   QString user = item->text(1);
+   QString host = item->text(0);
+   host = host.stripWhiteSpace();
+   user = user.stripWhiteSpace();
+   KeyList = new PatternList;
+   KeyList->push_back(CPatternPair(Host_Table,host));
+   KeyList->push_back(CPatternPair(User_Table,user));
+   return KeyList;
 }
 
 PatternList* CGrantTable::break_dbstring()
 {
-    int j = DBList->currentItem();
+    QListViewItem * item = DBList->selectedItem();
     PatternList*KeyList = 0;
-    if (j < 0)
+    if (!item)
 	return KeyList;
-    QString db = DBList->text(j);
-    j = db.find("\t");
-    if (j < 0) 
-	return KeyList;
-    QString host = db.left(j);
-    host = host.stripWhiteSpace();
-    db = db.mid(j+1, db.length());
-    j = db.find("\t");
-    if (j < 0)
-	return KeyList;
-    QString user = db.left(j);
+    QString db = item->text(2);
+    QString host = item->text(0);
+    QString user = item->text(1);
     user = user.stripWhiteSpace();
-    db = db.mid(j+1, db.length());
     db = db.stripWhiteSpace();
     KeyList = new PatternList;
-    //    KeyList->setAutoDelete(TRUE);
     KeyList->push_back(CPatternPair(Host_Table,host));
     KeyList->push_back(CPatternPair(User_Table,user));
     KeyList->push_back(CPatternPair(DB_Table,db));
@@ -794,17 +792,17 @@ void CGrantTable::Error()
     display_error_box();
 }
 
-void CGrantTable::hostselected(int)
+void CGrantTable::hostselected(QListViewItem*)
 {
     ModifyHost();
 }
 
-void CGrantTable::userselected(int)
+void CGrantTable::userselected(QListViewItem*)
 {
     ModifyUser();
 }
 
-void CGrantTable::dbselected(int)
+void CGrantTable::dbselected(QListViewItem*)
 {
     ModifyDb();
 }
@@ -817,3 +815,11 @@ void CGrantTable::UpdateGrants(int count)
 	sql_server->refresh_grants();
     }
 }
+
+void CGrantTable::FontChanged()
+{
+    QFont font = GlobalSettings.get_fixed_font();
+    HostList->setFont(font);
+    UserList->setFont(font);
+    DBList->setFont(font);
+}
