@@ -1,6 +1,6 @@
 #include <qstring.h>
 #include <qlist.h>
-#include <kmsgbox.h>
+//#include <kmsgbox.h>
 #include <kapp.h>
 #include <qmessagebox.h>
 
@@ -11,19 +11,32 @@
 #include "stringres.h"
 #include "my_sql.h"
 
+#include <qcheckbox.h>                                                          
+#include <qcombobox.h>                                                          
+#include <qframe.h>                                                             
+#include <qlabel.h>                                                             
+#include <qlcdnumber.h>                                                         
+#include <qlineedit.h>                                                          
+#include <qlistbox.h>                                                           
+#include <qpushbutton.h>                                                        
+#include <qradiobutton.h>                                                       
+#include <qslider.h>                                                            
+#include <qlayout.h>                                                            
+#include <qvariant.h>                                                           
+#include <qtooltip.h>                                                           
+#include <qwhatsthis.h>
+
 #ifndef rcsid
-static const char rcsid[]="$Id: modifytable.cpp,v 2.20 1999/11/07 16:12:50 ral Exp $";
+static const char rcsid[]="$Id: modifytable.cpp,v 2.26 2001/04/14 10:56:03 ral Exp $";
 //const char*modifytable_rcs(){return rcsid;}
 #endif
 
 #define Inherited CCreateTableData
 
-CModifyTable::CModifyTable(CMySql*_sql_server, QWidget* parent, const char* name )
-    :Inherited( parent, name )
+CModifyTable::CModifyTable(smart_pointer<CMySql>& _sql_server, QWidget* parent, const char* name )
+    :Inherited( parent, name, true)
 {
     sql_server = _sql_server;
-    QBoxLayout*main_layout = CHILD(this,QBoxLayout,"MainLayout");
-    TopMostLayout = CHILD(main_layout,QBoxLayout,"TopLineWidget");
     int count;
     for (count = 0; count < CMySql::FieldDescriptCounts(); count++) {
 	TypeComboList->insertItem(CMySql::FieldDescript(count));
@@ -40,10 +53,10 @@ CModifyTable::CModifyTable(CMySql*_sql_server, QWidget* parent, const char* name
 		DBList->insertItem(iter->c_str());
 	    }
 	} else {
-	    DBList->insertItem(i18n("No DBs found"));
+	    DBList->insertItem("No DBs found");
 	}
     } else {
-	DBList->insertItem(i18n("Not connected"));
+	DBList->insertItem("Not connected");
     }
 
     AddButton->setEnabled(FALSE);
@@ -213,26 +226,24 @@ void CModifyTable::SetTypeInt(int DataType)
 #undef STY
 }
 
-void CModifyTable::TableTextChanged(const char*text)
+void CModifyTable::TableTextChanged(const QString&text)
 {
-    bool how =  (text &&
-		 strlen(text) &&
-		 (CurrentTables.find(text) == 0) );
+    bool how =  (text.length()>0 &&
+		 (CurrentTables.find(text.latin1()) == 0) );
     ColumnNameEdit->setEnabled(how);
-    AddButton->setEnabled( how && strlen(ColumnNameEdit->text()) );
+    AddButton->setEnabled( how && ColumnNameEdit->text().length()>0 );
 }
  
 void CModifyTable::TableTextActivate()
 {
 }
 
-void CModifyTable::NameChanged(const char*text)
+void CModifyTable::NameChanged(const QString&text)
 {
-    bool how = ( text && strlen(text) );
-    AddButton->setEnabled(how);
+    AddButton->setEnabled(text.length()>0);
 }
 
-void CModifyTable::SetDBString(const char*text)
+void CModifyTable::SetDBString(const QString&)
 {
 }
 
@@ -335,6 +346,7 @@ int CModifyTable::ReadDescriptor(CColumnDescriptor*col_target,QString*str_target
 	int key_size = 0;
 
 	Key.set_name(Helper);
+        debug("Key name: %s\n",Key.get_name());
 	int Item;
 	if (sql_server->sql_type(DataType)._keysize) {
 	    Item = KeySizeSlider->value();
@@ -508,10 +520,10 @@ bool CModifyTable::_modify_column()
 		    return false;
 		}
 	    }
-	}
+	} else {
+        }
     }
-    
-    
+
     if (Desc != desc_list[i]) {
 	// check for duplicates if name has changed
 	if (desc_list.find(Desc)!=0) {
@@ -521,15 +533,14 @@ bool CModifyTable::_modify_column()
 	}
 	RemoveFromKey(desc_list[i].get_name(),true,Desc.get_name());
     }
-    
     RemoveFromKey(desc_list[i].get_name());
     if (!ReadDescriptor(&Desc,&text,&Key))
 	return false;
     desc_list[i]=Desc;
-    ColumnsList->changeItem(text,i);
     if (!Key_None->isChecked()) {
 	key_list.push_back(Key);
     }
+    ColumnsList->changeItem(text,i);
     return true;
 }
 
@@ -538,7 +549,6 @@ void CModifyTable::Delete_Column()
     int Index = ColumnsList->currentItem();
     int counter = 0;
     if (Index == -1) {
-	debug("index -1");
 	return;
     }
     CColumnDescriptor desc;
@@ -575,7 +585,7 @@ void CModifyTable::Build_Compound()
 
 void CModifyTable::AddEnum()
 {
-    if (enums.find(EnumList->currentText()))
+    if (enums.find(EnumList->currentText().latin1()))
 	return;
     QString current = EnumList->currentText();
     EnumList->insertItem(current,0);
@@ -587,7 +597,7 @@ void CModifyTable::DelEnum()
     if (EnumList->count() == 0)
 	return;
     int i = EnumList->currentItem();
-    stringIlist::iterator iter = enums.find(EnumList->currentText());
+    stringIlist::iterator iter = enums.find(EnumList->currentText().latin1());
     if (iter)
 	enums.erase(iter);
     if (i == -1)
@@ -613,3 +623,6 @@ void CModifyTable::RemoveFromKey(const char*col_name,bool compounds,const char*n
 	}
     }
 }
+void CModifyTable::Set_ValNotNull(bool)
+{
+}
