@@ -21,9 +21,10 @@
 
 #include "change_type.h"
 #include "specials.h"
+#include "stringhelper.h"
 
 #ifndef rcsid
-static const char rcsid[]="$Id: my_sql.cpp,v 2.38 1999/11/30 22:54:39 ral Exp $";
+static const char rcsid[]="$Id: my_sql.cpp,v 2.51 2001/04/23 22:48:30 ral Exp $";
 #endif
 
 const char* CMySql::MessageArray[]={
@@ -42,14 +43,14 @@ const my_sql_type CMySql::f_arr[]=
 {   //                                                             def  aut   zer   uns  key   keys  enums binary
     my_sql_type("TINYINT","TinyInt (1 Byte)",          1,  3,0,0, true, true, true, true,false, true,false,false),
     my_sql_type("SMALLINT","SmallInt (2 Bytes)",       1,  5,0,0, true, true, true, true,false, true,false,false),
-    my_sql_type("MEDIUMINT","MediumInt (3 Bytes",      1,  8,0,0, true, true, true, true,false, true,false,false),
+    my_sql_type("MEDIUMINT","MediumInt (3 Bytes)",     1,  8,0,0, true, true, true, true,false, true,false,false),
     my_sql_type("INT","Int (4 Bytes)",                 1, 10,0,0, true, true, true, true,false, true,false,false),
     my_sql_type("BIGINT","BigInt (8 Bytes)",           1, 20,0,0, true, true, true, true,false, true,false,false),
     my_sql_type("FLOAT","Float (4 Bytes)",             1, 20,0,4, true, true, true,false,false, true,false,false),
     my_sql_type("REAL","Double (8 Bytes)",             1, 20,0,8, true, true, true,false,false, true,false,false),
     my_sql_type("DECIMAL","Decimal (lenght,dec)",      1, 20,0,8, true, true, true,false, true, true,false,false),
-    my_sql_type("CHAR","Char (1 <= num <=255",         1,255,0,0,false,false,false,false, true, true,false, true),
-    my_sql_type("VARCHAR","Varchar (1 <= num <=255",   1,255,0,0,false,false,false,false, true, true,false, true),
+    my_sql_type("CHAR","Char (1 <= num <=255)",        1,255,0,0,false,false,false,false, true, true,false, true),
+    my_sql_type("VARCHAR","Varchar (1 <= num <=255)",  1,255,0,0,false,false,false,false, true, true,false, true),
     my_sql_type("TINYBLOB","Tinyblob (num <= 255)",    2,  2,0,0,false,false,false,false,false,false,false,false), //10
     my_sql_type("BLOB","Blob (num <= 65535)",          2,  2,0,0,false,false,false,false,false,false,false,false),
     my_sql_type("MEDIUMBLOB","Mediumblob (num < 2^24)",2,  2,0,0,false,false,false,false,false,false,false,false),
@@ -65,6 +66,8 @@ const my_sql_type CMySql::f_arr[]=
     my_sql_type("SET","Set",                           1, 80,0,0, true,false,false,false,false, true, true,false),
     my_sql_type("DATETIME","DateTime",                14, 14,0,0,false,false,false,false,false,false,false,false),
     my_sql_type("YEAR","Year",                         4,  4,0,0, true,false,false,false,false,false,false,false),
+    my_sql_type("DOUBLE","Double (8 Bytes)",           1, 20,0,8, true, true, true,false,false, true,false,false),
+    my_sql_type("NUMERIC","Decimal (lenght,dec)",      1, 20,0,8, true, true, true,false, true, true,false,false),
 };
 
 const my_sql_type&CMySql::sql_type(int i)
@@ -112,6 +115,7 @@ void CMySql::sql_disconnect()
 }
 
 CMySql::CMySql()
+    :refCounter()
 {
     Sql_Connection = 0L;
     nothing_else = 0;
@@ -120,6 +124,7 @@ CMySql::CMySql()
 }
 
 CMySql::CMySql(const CMySql&old)
+    :refCounter()
 {
     Sql_Connection = 0L;
     nothing_else = 0;
@@ -135,6 +140,12 @@ CMySql::~CMySql()
     sql_disconnect();
 }
 
+void CMySql::sql_connect(const std::string&host,const std::string&user,
+			 const std::string&pass,const std::string&port)
+{
+    sql_connect(host.c_str(),user.c_str(),pass.c_str(),port.c_str());
+}
+
 void CMySql::sql_connect(const char*host,const char*user,const char*pass,const char*port)
 {
     int Port;
@@ -153,13 +164,9 @@ void CMySql::sql_connect(const char*host,const char*user,const char*pass,const c
     Sql_Connection = new MYSQL;
     if (!Sql_Connection)
         return;
-#if MYSQL_VERSION_ID >= 32200
-    Sql_Connection = mysql_init(Sql_Connection);
-    Sql_Connection = mysql_real_connect(Sql_Connection,host,user,pass,(char*)0,(unsigned int)Port,(char*)0,0);
-#else
-    Sql_Connection = mysql_real_connect(Sql_Connection,host,user,pass,(unsigned int)Port,(char*)0,0);
-#endif
-    if (Sql_Connection) {
+    mysql_init(Sql_Connection);
+    MYSQL * dummy_pointer = mysql_real_connect(Sql_Connection,host,user,pass,(char*)0,(unsigned int)Port,(char*)0,0);
+    if (dummy_pointer) {
         do_message("Connected");
         s_version = mysql_get_server_info(Sql_Connection);
         if (s_version) {
@@ -174,11 +181,11 @@ void CMySql::sql_connect(const char*host,const char*user,const char*pass,const c
                 help_ptr = &help_ptr[1];
                 Server_PatchLevel = atoi(help_ptr);
             }
-	    //	    cout << "ServerMaster = "<<Server_MasterVersion<<"\nServerMinor = "<<Server_MinorVersion
-	    //		 << "\nPatchLevel = "<<Server_PatchLevel<<endl;
         }
     } else {
-        do_message("Couldnt connect");
+	do_message(FormatSTLString("Couldn't connect: %s",mysql_error(Sql_Connection)));
+	delete Sql_Connection;
+	Sql_Connection = 0;
     }
     nothing_else = 0;
 }
@@ -224,35 +231,27 @@ const std::string&CMySql::get_info()
     char*h_info;
     int p_info;
     char*s_info;
-    char*all_info;
-    char*t_1 = "Clientversion";
+    char t_1[] = "Clientversion";
     if (nothing_else) {
-        all_info = new char[1];
-        strcpy(all_info,"");
+        connect_info = "";
     } else {
         if (Sql_Connection) {
             c_info = mysql_get_client_info();
             h_info = mysql_get_host_info(Sql_Connection);
             p_info = mysql_get_proto_info(Sql_Connection);
             s_info = mysql_get_server_info(Sql_Connection);
-            all_info = new char[strlen(c_info)+strlen(h_info)+strlen(s_info)+strlen(t_1)+50];
 #ifndef WIN32
-            sprintf(all_info,
-		    "%s\n%s %s\nProtokoll Version: %i\nServerversion: %s",
+            connect_info = FormatSTLString("%s\n%s %s\nProtokoll Version: %i\nServerversion: %s",
 		    h_info,t_1,c_info,p_info,s_info);
 #else
-            sprintf(all_info,
-		    "%s\r\n%s %s\r\nProtokoll Version: %i\r\nServerversion: %s",
+            connect_info = FormatSTLString("%s\r\n%s %s\r\nProtokoll Version: %i\r\nServerversion: %s",
 		    h_info,t_1,c_info,p_info,s_info);
 #endif
         }
         else {
-            all_info = new char[19];
-            strcpy(all_info,"Couldn't get info");
+            connect_info = "Couldn't get info";
         }
     }
-    connect_info = all_info;
-    delete[]all_info;
     return connect_info;
 }
 
@@ -287,7 +286,7 @@ bool CMySql::get_threads(stringlist&target)const
     result = "";
     while ((row=mysql_fetch_row(Res))) {
         for (unsigned int i = 0;i < mysql_num_fields(Res);++i) {
-            result+=row[i]!=0?(char*)row[i]:"";
+            result+=row[i]!=0?(char*)row[i]:" ";
             if(i < mysql_num_fields(Res) - 1)
                 result+="\t";
         }
@@ -298,6 +297,33 @@ bool CMySql::get_threads(stringlist&target)const
     return true;
 }
 
+bool CMySql::get_threads(list_vector&target)const
+{
+    MYSQL_RES*Res;
+    MYSQL_ROW row;
+    target.resize(0);
+    if (!Sql_Connection || nothing_else)
+        return true;
+    Res = mysql_list_processes(Sql_Connection);
+    const char*st;
+    st = mysql_error(Sql_Connection);
+    if (st[0] != 0) {
+        do_message(st);
+    }
+    if (!Res)
+        return false;
+    stringlist result;
+    while((row=mysql_fetch_row(Res))) {
+        for (unsigned int i = 0; i < mysql_num_fields(Res);++i) {
+            result.push_back(row[i]!=0?row[i]:"");
+        }
+        target.push_back(result);
+        result.clear();
+    }
+    mysql_free_result(Res);
+    return true;
+}
+
 /*
 * retrieves the list of databases in the current
 * connected server
@@ -333,12 +359,10 @@ bool CMySql::get_db_list(const char*pattern,stringlist&target)const
 }
 
 /*
-* fetches the tables of a database
-*/
-//stringlist*
+ * fetches the tables of a database
+ */
 bool CMySql::get_table_list(const char*pattern,const char*db,stringlist&target)
 {
-    //    stringlist*ResultList = 0;
     std::string s;
     MYSQL_RES*Res;
     MYSQL_ROW row;
@@ -355,7 +379,6 @@ bool CMySql::get_table_list(const char*pattern,const char*db,stringlist&target)
     }
     if (!Res)
         return false;
-    //    ResultList = new stringlist;
     while ((row=mysql_fetch_row(Res))) {
         for (unsigned int i = 0;i < mysql_num_fields(Res);i++) {
             s+=row[i]?(char*)row[i]:"";
@@ -386,16 +409,15 @@ bool CMySql::get_table_fields(const char*db,const char*table,ColumnList&target)c
         return false;
 
     if (!db) {
-	do_message(MessageArray[6]);
-	return false;
+		do_message(MessageArray[6]);
+		return false;
     }
     if (connect_db(db)!=0) {
         do_message("Could not connect to database");
-        return -1;
+        return false;
     }
 
     command = "SHOW COLUMNS FROM ";
-    //command+=db;command+='.';
     command+=table;
     int i = mysql_query(Sql_Connection,command.c_str());
     if (i < 0) {
@@ -430,6 +452,11 @@ bool CMySql::get_table_fields(const char*db,const char*table,ColumnList&target)c
 		break;
 	    }
 	}
+	if (f_type == FieldDescriptCounts()) {
+		do_message("Error retrieving fieldtypes - please send a mail to the "
+				"autor of the software!");
+		return false;
+	}
 	my_sql_type t_info = sql_type(f_type);
 	if (f_type > 10 && f_type < 18) {
 	    entry.set_Blob(true);
@@ -556,8 +583,7 @@ bool CMySql::create_table(const char*db,const char*name,ColumnList*columns,KeyLi
     KeyList_iterator key = NULL;
     unsigned int count = 0;
     std::string Type;
-    char buffer[128];
-    memset(buffer,0,128);
+
     if (!name || strlen(name) == 0) {
         do_message(MessageArray[3]);
         return false;
@@ -573,7 +599,7 @@ bool CMySql::create_table(const char*db,const char*name,ColumnList*columns,KeyLi
     
     if (connect_db(db)!=0) {
         do_message("Could not connect to database");
-        return -1;
+        return false;
     }
 
     if (!db) {
@@ -637,9 +663,7 @@ bool CMySql::create_table(const char*db,const char*name,ColumnList*columns,KeyLi
                 statement+=iter->first.c_str();
                 if (iter->second > 0) {
                     statement+="(";
-                    memset(buffer,0,128);
-                    sprintf(buffer,"%u",iter->second);
-                    statement+=buffer;
+                    statement+= FormatSTLString("%u",iter->second);
                     statement+=")";
                 }
             }
@@ -647,7 +671,7 @@ bool CMySql::create_table(const char*db,const char*name,ColumnList*columns,KeyLi
         }
     }
     statement+=")";
-    do_message(statement.c_str());
+    do_message(statement);
     int i = mysql_query(Sql_Connection,statement.c_str());
     if (i < 0) {//error??
         //why?
@@ -657,44 +681,24 @@ bool CMySql::create_table(const char*db,const char*name,ColumnList*columns,KeyLi
     return true;
 }
 
-/*
- * Makes a simple search
- */
-int CMySql::fetch_rows(const char*db,const char*table,const char*sort_by,
-                       stringlist*which,unsigned int offset,unsigned int count,
-                       PatternList*Pattern,stringlist*sizes)
+int CMySql::_fetch_rows(const char*db,const char*table,const char*sort_by,
+                   stringlist*which,unsigned int offset,unsigned int count,
+                   PatternList*Pattern,bool errors_only,MYSQL_RES**Res)
 {
     std::string query;
     std::string entry;
-    MYSQL_RES*Res;
-    MYSQL_ROW row;
-    MYSQL_FIELD*Field;
-    char buffer[512];
-    char*escaped;
-    
-    Field = 0;
+    std::string escaped;
+#ifndef WIN32
+    timeval starttime,endtime;
+#endif
+
     int i = 0;
-    memset(buffer,0,512);
-    unsigned int j;
-    if (!which) {
-        do_message("Which rows?");
-        return -1;
-    }
-    if (!Sql_Connection) {
-        do_message(MessageArray[4]);
-        return -1;
-    }
-    if (!table || strlen(table) == 0) {
-        do_message(MessageArray[3]);
-        return -1;
-    }
-    if (!db) {
-	do_message(MessageArray[6]);
-	return -1;
-    }
-    i = connect_db(db);
-    if (i < 0)
-        return i;
+
+    if (!which) {do_message("Which rows?");return -1;}
+    if (!Sql_Connection) {do_message(MessageArray[4]);return -1;}
+    if (!table || strlen(table) == 0) {do_message(MessageArray[3]);return -1;}
+    if (!db) {do_message(MessageArray[6]);return -1;}
+    if (connect_db(db)<0) {return -1;}
 
     query = "";
     stringlist::iterator w_iter = which->begin();
@@ -720,20 +724,18 @@ int CMySql::fetch_rows(const char*db,const char*table,const char*sort_by,
         query+=MessageArray[2];
         //    query+=" ";
         query+=p_iter->get_entry();query+=" LIKE \'";
-        escaped = do_escape(p_iter->get_pattern());
-        if (!escaped)
+        do_escape(p_iter->get_pattern(),escaped);
+        if (escaped.size()==0)
             return -1;
         query+=escaped;
-        delete[] escaped;
         query+="\' ";
         ++p_iter;
         for (;p_iter != Pattern->end();++p_iter){
             query+="AND ";query+=p_iter->get_entry();query+=" LIKE \'";
-            escaped = do_escape(p_iter->get_pattern());
-            if (!escaped)
+            do_escape(p_iter->get_pattern(),escaped);
+            if (escaped.size()==0)
                 return -1;
             query+=escaped;
-            delete[] escaped;
             query+="\' ";
         }
     }
@@ -746,9 +748,7 @@ int CMySql::fetch_rows(const char*db,const char*table,const char*sort_by,
         std::string ls;
         std::string cs;
         if (offset > 0) {
-            sprintf(buffer,"LIMIT %u",offset);
-            ls = buffer;
-            memset(buffer,0,512);
+            ls = FormatSTLString("LIMIT %u",offset);
             if (count > 0) {
                 ls+=",";
             }
@@ -759,9 +759,7 @@ int CMySql::fetch_rows(const char*db,const char*table,const char*sort_by,
             ls = "LIMIT ";
         }
         if (count > 0) {
-            sprintf(buffer,"%u",count);
-            ls+=buffer;
-            memset(buffer,0,512);
+            ls+= FormatSTLString("%u",count);
         }
         query+=ls;
     }
@@ -770,15 +768,19 @@ int CMySql::fetch_rows(const char*db,const char*table,const char*sort_by,
     if (Pattern) 
         Pattern->clear();
     //what is the resulting query?
-    do_message(query.c_str());
-    
+    if (!errors_only) do_message(query.c_str());
+#ifndef WIN32    
+    gettimeofday(&starttime,0);
+#endif
     i = mysql_query(Sql_Connection,query.c_str());
+
     if (i < 0) {//error??
         //why?
+	if (errors_only)do_message(query);
         do_message(get_last_error());
         return -1;
     }
-    Res = mysql_store_result(Sql_Connection);
+    *Res = mysql_store_result(Sql_Connection);
     if (!Res) {
         //autch - this should never be...
         query = mysql_error(Sql_Connection);
@@ -788,16 +790,46 @@ int CMySql::fetch_rows(const char*db,const char*table,const char*sort_by,
             do_message("Error fetch the result");
         return -1;
     }
+#ifndef WIN32
+    gettimeofday(&endtime,0);
+    long int sec,usec;
+    sec = endtime.tv_sec - starttime.tv_sec;
+    usec = endtime.tv_usec - starttime.tv_usec;
+    usec += 1000*1000*sec;
+    printf("Fetchtime: %li,%06li sec ",usec/(1000*1000),usec%(1000*1000));
+#endif
+    return 1;
+}
+
+/*
+ * Makes a simple search
+ */
+int CMySql::fetch_rows(const char*db,const char*table,const char*sort_by,
+                       stringlist*which,unsigned int offset,unsigned int count,
+                       PatternList*Pattern,bool errors_only)
+{
+    MYSQL_RES*Res;
+    int i = _fetch_rows (db,table,sort_by,which,offset,count,Pattern,errors_only,&Res);
+    if (i < 0)
+        return -1;
+    unsigned j = 0;
+    MYSQL_ROW row;
+    MYSQL_FIELD*Field;
+
     //what did the server found?
-    
     unsigned long zaehler = Res->row_count;
-    sprintf(buffer,"%lu %s",zaehler,MessageArray[1]);
-    do_message(buffer);
-    entry.resize(0);
+    if (!errors_only) do_message(FormatSTLString("%lu %s",zaehler,MessageArray[1]));
+    std::string entry,query;
+    entry = "";query = "";
     std::string helper = "";
+    int groesse;
+#ifndef WIN32
+    timeval starttime,endtime;
+    long int sec,usec;
+    gettimeofday(&starttime,0);
+#endif
     while ((row=mysql_fetch_row(Res))) {
-        for (j = 0;j < mysql_num_fields(Res);j++) {     
-	    query.resize(0);
+        for (j = 0;j < mysql_num_fields(Res);++j) {
             query = row[j]?(char*)row[j]:"";
             //how long is the longest?
             Res->current_field = j;
@@ -805,10 +837,10 @@ int CMySql::fetch_rows(const char*db,const char*table,const char*sort_by,
             // we retrieve the first line splitted, for request, which will only need this line
             // for instance a query to the user rights
             if (which->size() == 0) {
-                if (sizes) {
-                    sprintf(buffer,"%i",Field->max_length);
-                    sizes->push_back(std::string(buffer));
+                /*                if (sizes) {
+                    sizes->push_back(FormatSTLString("%i",Field->max_length));
                 }
+                */
                 if ( Pattern ) {
                     Pattern->push_back(CPatternPair(Field->name,query));
                 }
@@ -816,8 +848,10 @@ int CMySql::fetch_rows(const char*db,const char*table,const char*sort_by,
             //ok - justify
             helper.resize(0);
             if (IS_NUM(Field->type)) {
-		int groesse = Field->max_length-query.size();
+		groesse = Field->max_length-query.size();
 		helper.resize(groesse>=0?groesse:0,' ');
+            } else {
+                helper = "";
             }
 	    helper+=query;
             if (helper.size() == 0)
@@ -831,22 +865,91 @@ int CMySql::fetch_rows(const char*db,const char*table,const char*sort_by,
         which->push_back(entry);
         entry = "";
     }
+#ifndef WIN32
+    gettimeofday(&endtime,0);
+    sec = endtime.tv_sec - starttime.tv_sec;
+    usec = endtime.tv_usec - starttime.tv_usec;
+    usec+=1000*1000*sec;
+    printf("Listbuildtime: %li,%06li sec\n",usec/(1000*1000),usec%(1000*1000));
+#endif
     mysql_free_result(Res);
     
     return 0;
 }
 
 /*
-* inserts a single row into a table
-*/
+ * Makes a simple search
+ */
+int CMySql::fetch_rows(const char*db,const char*table,const char*sort_by,
+                       stringlist*which,list_vector&target,unsigned int offset,unsigned int count,
+                       PatternList*Pattern,bool errors_only)
+{
+    MYSQL_RES*Res;
+    int i = _fetch_rows (db,table,sort_by,which,offset,count,Pattern,errors_only,&Res);
+    if (i < 0)
+        return -1;
+    unsigned j = 0;
+    MYSQL_ROW row;
+    MYSQL_FIELD*Field;
+
+    //what did the server found?
+    unsigned long zaehler = Res->row_count;
+    if (!errors_only) do_message(FormatSTLString("%lu %s",zaehler,MessageArray[1]));
+    std::string query;
+    query = "";
+    stringlist helper,entry;
+#ifndef WIN32
+    timeval starttime,endtime;
+    long int sec,usec;
+    gettimeofday(&starttime,0);
+#endif
+    target.resize(0);
+    entry.resize(0);
+    while ((row=mysql_fetch_row(Res))) {
+        for (j = 0;j < mysql_num_fields(Res);++j) {
+            query = row[j]?(char*)row[j]:"";
+            //how long is the longest?
+            Res->current_field = j;
+            Field = mysql_fetch_field(Res);
+            // we retrieve the first line splitted, for request, which will only need this line
+            // for instance a query to the user rights
+            if (which->size() == 0) {
+                /*                if (sizes) {
+                    sizes->push_back(FormatSTLString("%i",Field->max_length));
+                    }*/
+                if ( Pattern ) {
+                    Pattern->push_back(CPatternPair(Field->name,query));
+                }
+            }
+            entry.push_back(query);
+        }
+        //append to the list
+        target.push_back(entry);
+        entry.resize(0);
+    }
+#ifndef WIN32
+    gettimeofday(&endtime,0);
+    sec = endtime.tv_sec - starttime.tv_sec;
+    usec = endtime.tv_usec - starttime.tv_usec;
+    usec+=1000*1000*sec;
+    printf("Listbuildtime: %li,%06li sec\n",usec/(1000*1000),usec%(1000*1000));
+#endif
+    mysql_free_result(Res);
+    
+    return 0;
+}
+
+
+/*
+ * inserts a single row into a table
+ */
 int CMySql::insert_row(const char*db,const char*table, PatternList*Pattern)
 {
     std::string query = "";
     std::string value_string = "";
     std::string row_string = "";
-    char Buffer[256];
-    //    CPatternPair*pat = 0;
-    char*escaped;
+
+    std::string escaped;
     int i;
     if (!Sql_Connection) {
         do_message(MessageArray[4]);
@@ -870,21 +973,19 @@ int CMySql::insert_row(const char*db,const char*table, PatternList*Pattern)
     for (p_iter = Pattern->begin();p_iter!=Pattern->end();++p_iter){
         if (value_string.empty()) {
             value_string = "\'";
-            escaped = do_escape(p_iter->get_pattern());
-            if (!escaped)
+            do_escape(p_iter->get_pattern(),escaped);
+            if (!escaped.size())
                 return -1;
             value_string += escaped;
-            delete[]escaped;
             value_string+="\'";
             row_string = p_iter->get_entry();
         }
         else {
             value_string+=",\'";
-            escaped = do_escape(p_iter->get_pattern());
-            if (!escaped)
+            do_escape(p_iter->get_pattern(),escaped);
+            if (!escaped.size())
                 return -1;
             value_string+=escaped;
-            delete[]escaped;
             value_string+="\'";
             row_string+=",";row_string+=p_iter->get_entry();
         }
@@ -906,41 +1007,39 @@ int CMySql::insert_row(const char*db,const char*table, PatternList*Pattern)
     }
     //should be > 0 
     i = mysql_affected_rows(Sql_Connection);
-    memset(Buffer,0,256);
-    sprintf(Buffer,"%i %s",i,MessageArray[0]);
-    //log it
-    do_message(Buffer);
+    do_message(FormatSTLString("%i %s",i,MessageArray[0]));
     return i;
 }
 
-char*CMySql::do_escape(const char*what)
+std::string&CMySql::do_escape(const std::string&what,std::string&target)
 {
-    char*of1;
-    char*escaped = new char[strlen(what)*2+1];
-    if (!what)
-        return 0;
+    target.resize(0);
+    string _s = "%%";
+    string _r = "\\%";
+
+    if (what.size()==0)
+        return target;
+
+    char * escaped = new char[what.size()*2+1];
     if (!escaped) {
         do_message("Error allocating memory");
-        return 0;
+        return target;
     }
-    memset(escaped,'\0',strlen(what)*2+1);
-    mysql_escape_string(escaped,(char*)what,strlen(what));
-    of1 = strstr(escaped,"%%");
-    while (of1) {
-        of1[0] = '\\';
-        //we don't hack "%%%" -> "\%\%" but -> "\%%"
-        of1 = &of1[2];
-        of1 = strstr(of1,"%%");
+    memset(escaped,'\0',what.size()*2+1);
+    mysql_escape_string(escaped,what.c_str(),what.size());
+    target = escaped;
+
+    string::size_type it = target.find(_s);
+
+    while (it!=string::npos) {
+        target.replace(it,1,_r);
+        cout << "target: " << target.c_str()<< endl;
+        ++it;++it;++it;
+        it = target.find(_s,it);
     }
-    of1 = new char [strlen(escaped)+1];
-    if (!of1) {
-        do_message("Error allocating memory");
-        return 0;
-    }
-    memset(of1,'\0',strlen(escaped)+1);
-    strcpy(of1,escaped);
-    delete[]escaped;
-    return of1;
+    if (escaped)
+        delete[]escaped;
+    return target;
 }
 
 int CMySql::update_row(const char*db,const char*table,
@@ -949,10 +1048,8 @@ int CMySql::update_row(const char*db,const char*table,
 {
     std::string request;
     std::string clean;
-    //    CPatternPair*pattern;
     int i;
-    char Buffer[256];
-    memset(Buffer,0,256);
+
     if (!Sql_Connection) {
         do_message(MessageArray[4]);
         return -1;
@@ -980,7 +1077,7 @@ int CMySql::update_row(const char*db,const char*table,
         if (p_iter != to_change->begin())
             request+=",";
         request+=p_iter->get_entry();request+="=\'";
-        clean = do_escape(p_iter->get_pattern());
+        do_escape(p_iter->get_pattern(),clean);
         request+=clean;
         request+="\'";
     }
@@ -990,7 +1087,7 @@ int CMySql::update_row(const char*db,const char*table,
         if (p_iter != identifiers->begin())
             request += " AND ";
         request+=p_iter->get_entry();request+=" = \'";
-        clean = do_escape(p_iter->get_pattern());
+        do_escape(p_iter->get_pattern(),clean);
         request+=clean;
         request+="\'";
     }
@@ -1004,8 +1101,7 @@ int CMySql::update_row(const char*db,const char*table,
     //should be >= 0 
     i = mysql_affected_rows(Sql_Connection);
     //log it
-    sprintf(Buffer,"%i %s",i,MessageArray[0]);
-    do_message(Buffer);
+    do_message(FormatSTLString("%i %s",i,MessageArray[0]));
     return i;
 }
 
@@ -1013,12 +1109,10 @@ int CMySql::delete_row(const char*db,const char*table,
                        PatternList*identifiers)
 {
     std::string query;
-    char* clean;
-    //    CPatternPair*pattern;
+    std::string clean;
     int i;
     unsigned long zaehler;
-    char buffer[512];
-    memset(buffer,0,512);
+
     if (!Sql_Connection){
         do_message(MessageArray[4]);
         return -1;
@@ -1047,17 +1141,16 @@ int CMySql::delete_row(const char*db,const char*table,
             if (p_iter != identifiers->begin())
                 query+=" AND ";
             query+=p_iter->get_entry();query+=" = \'";
-            clean = do_escape(p_iter->get_pattern());
-            if (!clean) {
+            do_escape(p_iter->get_pattern(),clean);
+            if (!clean.size()) {
                 do_message("error escaping strings");
                 return -1;
             }
             query+=clean;
-            delete[]clean;
             query+="\'";
         }    
     }
-    do_message(query.c_str());
+    do_message(query);
     i = mysql_query(Sql_Connection,query.c_str());
     if (i == -1) {
         //if some goes wrong 
@@ -1067,8 +1160,7 @@ int CMySql::delete_row(const char*db,const char*table,
     //should be > 0 
     zaehler = mysql_affected_rows(Sql_Connection);
     //log it
-    sprintf(buffer,"%lu %s",zaehler,MessageArray[0]);
-    do_message(buffer);
+    do_message(FormatSTLString("%lu %s",zaehler,MessageArray[0]));
     return zaehler;
 }
 
@@ -1112,8 +1204,7 @@ int CMySql::shutdown()
 bool CMySql::delete_table(const char*db, const char*table)
 {
     std::string query;
-    char buffer[64];
-    memset(buffer,0,64);
+
     if (!db || !table) {
         do_message("delete_table: To few parameters");
         return false;
@@ -1134,9 +1225,7 @@ bool CMySql::delete_table(const char*db, const char*table)
         return false;
     }
     unsigned long zaehler = mysql_affected_rows(Sql_Connection);
-    sprintf(buffer,"%lu %s",zaehler,MessageArray[0]);
-    //log it
-    do_message(buffer);
+    do_message(FormatSTLString("%lu %s",zaehler,MessageArray[0]));
     return true;
 }
 
@@ -1146,7 +1235,6 @@ bool CMySql::get_table_keys(const char*db, const char*table, KeyList&target)cons
     std::string name="";
     MYSQL_RES*Res=0;
     MYSQL_ROW row;
-    //    MYSQL_FIELD*Field=0;
 	
     target.resize(0);
     if (!Sql_Connection) {
@@ -1236,7 +1324,6 @@ bool CMySql::rename_table(const char*db,const char*table,const char*new_name)con
 	return false;
     }
     command = "ALTER TABLE ";
-    //command+=db;command+='.';
     command+= table;command+= " RENAME AS ";command+= new_name;
     do_message(command);
     int i = mysql_query(Sql_Connection,command.c_str());
@@ -1273,7 +1360,7 @@ bool CMySql::alter_table(const char*db, const char*table,ColumnList&n_columns,co
     }
     if (connect_db(db)!=0) {
         do_message("Could not connect to database");
-        return -1;
+		return false;
     }
     command = "ALTER TABLE ";
     //command+=db;command+='.';
@@ -1359,7 +1446,6 @@ bool CMySql::alter_table(const char*db, const char*table,ColumnList&n_columns,co
 
 int CMySql::build_command(std::string&target,CColumnDescriptor&desc)const
 {
-    char buffer[128];
     std::string Type = "";
     target.resize(0);
     target=desc.get_name();
@@ -1383,8 +1469,7 @@ int CMySql::build_command(std::string&target,CColumnDescriptor&desc)const
 	    max_enum = 65535;
 	}
 	if (current_enums.size() > max_enum) {
-	    sprintf(buffer,"More than %u %ss are not allowed",max_enum,max_enum==64?"SET":"ENUM");
-	    do_message(buffer);
+	    do_message(FormatSTLString("More than %u %ss are not allowed",max_enum,max_enum==64?"SET":"ENUM"));
 	    return 0;
 	}
 	Type+="(";
@@ -1400,16 +1485,11 @@ int CMySql::build_command(std::string&target,CColumnDescriptor&desc)const
     target+=Type;
     target+=" ";
     if (desc.get_MaxLength()>0 && desc.get_FieldType() != 21 && desc.get_FieldType() != 22) {
-	char buffer[10];
 	target+="(";
-	sprintf(buffer,"%i",desc.get_MaxLength());
-	//        Type = Type.setNum(desc->get_MaxLength());
-	target+=buffer;
-		
+	target+=FormatSTLString("%i",desc.get_MaxLength());
 	if (desc.get_MaxDecimals() > 0) {
-	    sprintf(buffer,"%i",desc.get_MaxDecimals());
 	    target+=", ";
-	    target+=buffer;
+	    target+=FormatSTLString("%i",desc.get_MaxDecimals());
 	}
 	target+=") ";
     }
@@ -1436,7 +1516,6 @@ int CMySql::build_key_command(std::string&target,const KeyList*keys,bool add)
 {
     target.resize(0);
     KeyList::const_iterator key;
-    char buffer[128];
     if (keys) {
         int already_prim = 0;
         pairIlist col_list;
@@ -1483,9 +1562,7 @@ int CMySql::build_key_command(std::string&target,const KeyList*keys,bool add)
                 target+=iter->first.c_str();
                 if (iter->second > 0) {
                     target+="(";
-                    memset(buffer,0,128);
-                    sprintf(buffer,"%u",iter->second);
-                    target+=buffer;
+                    target+=FormatSTLString("%u",iter->second);
                     target+=")";
                 }
             }
@@ -1509,7 +1586,7 @@ bool CMySql::do_statement(const char*statement,stringlist&target,stringlist&size
     MYSQL_FIELD*Field;
 
     std::string query;
-    char buffer[100];
+
     MYSQL_RES*Res=0;
     if (!log_errors_only)
 	do_message(statement);
@@ -1532,9 +1609,8 @@ bool CMySql::do_statement(const char*statement,stringlist&target,stringlist&size
         return true;
     }
     unsigned long zaehler = Res->row_count;
-    sprintf(buffer,"%lu %s",zaehler,MessageArray[0]);
     if (!log_errors_only)
-	do_message(buffer);
+	do_message(FormatSTLString("%lu %s",zaehler,MessageArray[0]));
     std::string helper;
     std::string entry;
     unsigned int j;
@@ -1556,8 +1632,6 @@ bool CMySql::do_statement(const char*statement,stringlist&target,stringlist&size
 	    }
             if (target.size() == 0) {
 		headers.push_back(std::string(f));
-		//		sprintf(buffer,"%i",max_size);
-		//		sizes.push_back(std::string(buffer));
             }
             //ok - justify
 	    int groesse = max_size-query.size();
@@ -1567,7 +1641,7 @@ bool CMySql::do_statement(const char*statement,stringlist&target,stringlist&size
 	    }
 	    helper+=query;
 	    if (!IS_NUM(Field->type)) {
-		helper.resize(max_size>=0?max_size:query.size(),' ');
+		helper.resize(max_size,' ');
 	    }
             if (helper.size() == 0)
                 helper = " ";
