@@ -10,6 +10,8 @@
 #include "sendmessage.h"
 #include "kstatusbar.h"
 #include <qtimer.h>
+#include "smart_pointer.h"
+#include "edit.h"
 
 class CMySql;
 
@@ -18,7 +20,7 @@ class CSqlScript : public CSqlScriptData, virtual public CSendMessage
     Q_OBJECT
 public:
  
-    CSqlScript( CMySql*_sql_server,QWidget* parent = NULL,const char* name = NULL );
+    CSqlScript(smart_pointer<CMySql>& _sql_server,QWidget* parent = NULL,const char* name = NULL );
     virtual ~CSqlScript();
     virtual void append_message(const char*text);
     virtual void append_message(const std::string&message);
@@ -36,6 +38,7 @@ protected slots:
     virtual void searchAgain();
     void keyPressEvent ( QKeyEvent * e );
     void set_cursor_info();
+    void FixFontChanged();
 
 protected:
     std::string ResultText;
@@ -44,12 +47,13 @@ protected:
     std::string search_text;
     
     void search_forward();
-    CMySql*sql_server;
+    smart_pointer<CMySql> sql_server;
     void split_commands(stringlist & target);
     void setStatusText(const char*text,unsigned int timeout=0);
     int last_tab;
     QMenuBar*MenuBar;
     QTimer*status_Timer;
     void save_string(std::string*contents,bool textonly);
+    CEdit * currentWindow();
 };
 #endif // CSqlScript_included
